#pragma once
#include <JuceHeader.h>
using namespace juce;

class MainComponent :
  public Component,
  Button::Listener,
  Thread {
public:
  MainComponent(): Thread("MainComponent") {
    mDeviceNames.add("01. Internal MIDI");
    mDeviceNames.add("02. Internal MIDI");
    mDeviceNames.add("03. Internal MIDI");
    // The problem is the same with physical MIDI output ports
    // like the commented out ones below
    // as it is with the internal/virtual ports above.
    // mDeviceNames.add("KOMPLETE KONTROL - 1");
    // mDeviceNames.add("KOMPLETE KONTROL EXT - 1");
    // mDeviceNames.add("2- Launch Control XL");
    
    runButton.setButtonText("Run");
    runButton.addListener(this);
    addAndMakeVisible(runButton);

    messageLogLabel.setFont(20);
    messageLogLabel.setJustificationType(Justification::topLeft);
    addAndMakeVisible(messageLogLabel);
    
    setSize(800, 400);
  }

  ~MainComponent() override {
    if (isThreadRunning()) {
      signalThreadShouldExit();
    }
  }

  void buttonClicked(Button* button) override {
    if (button != &runButton) {
      return;
    }
    messageLogLabel.setText(String(), dontSendNotification);
    startThread();
  }

  void logMessage(const StringRef text) {
    DBG(text);
    MessageManagerLock lock;
    messageLogLabel.setText(
      messageLogLabel.getText() + "\r\n" + text, dontSendNotification);
  }

  void resized() override {
    constexpr int margin = 10;
    const int width = getWidth() - 2 * margin; 
    runButton.setBounds(
      margin,
      margin,
      width,
      36);
    messageLogLabel.setBounds(
      margin,
      runButton.getBottom() + margin,
      width,
      getLocalBounds().getBottom() - runButton.getBottom() - margin);
  }

  void run () override {
    for (const auto& deviceName : mDeviceNames) {
      if (!tryOpenMidiOutput(deviceName)) {
        return;
      }
    }
  }

  void showErrorMessage(const StringRef text) {
    logMessage(text);
    AlertWindow::showMessageBoxAsync(
      AlertWindow::WarningIcon,
      ProjectInfo::projectName,
      text,
      "OK");
  }

  bool tryOpenMidiOutput(const StringRef deviceName) {
    bool foundDevice = false;
    for (const auto& device : MidiOutput::getAvailableDevices()) {
      if (device.name == deviceName) {
        foundDevice = true;
        break;
      }
    }
    if (!foundDevice) {
      showErrorMessage("Cannot find MIDI output device '" + deviceName + "'.");
      return false;
    }
    logMessage("Found MIDI output device '" + deviceName + "'.");
    mMidiOutputs.push_back(MidiOutput::openDevice(deviceName));
    if (mMidiOutputs.operator[](mMidiOutputs.size() - 1) == nullptr) {
      showErrorMessage("MIDI output device '" + deviceName +
        "' has been found but could not be opened.");
      return false;
    }
    logMessage("Opened MIDI output device '" + deviceName + "'.");
    return true;
  }
private:
  TextButton runButton;
  Label messageLogLabel;
  
  StringArray mDeviceNames;
  std::vector<std::unique_ptr<MidiOutput>> mMidiOutputs;

  JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(MainComponent)
};
